#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "vroot.h"

#ifdef sys_errlist
#undefine sys_errlist
extern char *sys_errlist[];
#endif

#define NUMBERofERRORS ENOSYS+1
#define MERGIN 5
#define FONTNAME "-b&h-*-bold-r-*-*-%d-*-*-*-*-*-*-*"

int sizes[] = {60, 50, 40, 30, 20, 10}; 

Display *disp;
Window win;
int win_width, win_height;
int screen;
GC gc;
XFontStruct *font_info;

#define OPT_ROOT "-root"
#define OPT_DELAY "-delay"
#define OPT_DISP "-display"
#define MODE_ROOT    001

#define DEFAULT_MODE 0
#define DEFAULT_DELAY 64
#define DEFAULT_WAIT 3

char *options[] = { OPT_ROOT, OPT_DELAY, OPT_DISP, NULL };
enum options { OPT_ROOT_NUM, OPT_DELAY_NUM, OPT_DISP_NUM};

int mode = DEFAULT_MODE;
int delay = DEFAULT_DELAY;
int waitsec = DEFAULT_WAIT;
char *dispname = NULL;

void DoOneTurn();

main(argc, argv)
int argc;
char *argv[];
{
    int  i, j, k;
    char fontname[100];

    srandom((int)(time((long *) NULL)%1000 + getpid()));

    /* options */
    i = 1;
    while( i!= argc ){
	j=0;
	while( options[j] != NULL ){
	    if( !strcmp( options[j], argv[i] ) ){
		switch( j ){
		case OPT_ROOT_NUM:
		    mode |= MODE_ROOT;
		    break;
		case OPT_DELAY_NUM:
		    i++;
		    if( i == argc ){
			fprintf( stderr, "-delay need integar.\n" );
			error_exit();
		    }
		    if(( delay = atoi( argv[i] ) ) == 0 ){
			fprintf( stderr, "-delay need non-zero integar.\n");
			error_exit();
		    }
		    break;
		case OPT_DISP_NUM:
		    i++;
		    if( i== argc ){
			fprintf( stderr, "-display need string.\n");
			error_exit();
		    }
		    dispname = argv[i];
		    break;
		default:
		    fprintf( stderr, "unknown option %s\n", argv[i] );
		    error_exit();
		}
		i++;
		break;
	    } else j++;
	}
    }

/*
  $B%G%#%9%W%l%$$N%*!<%W%s$J$I(B
*/
    disp = XOpenDisplay(dispname);
    screen = DefaultScreen( disp ); 
    gc = XCreateGC(disp, RootWindow(disp, screen), 0, NULL);
  
/*
  $B%U%)%s%H$N<hF@(B
*/

    sprintf( fontname, FONTNAME, 60 );
    if((font_info = XLoadQueryFont ( disp, fontname ))== NULL){
	fprintf( stderr, "xerror : can't load font %s.\n", FONTNAME );
	exit( 1 );
    }
    XSetFont( disp, gc, font_info->fid );

/*
  $B%H%C%W%l%Y%k%&%#%s%I%&$N:n@.(B
*/
    if( mode & MODE_ROOT )
	win = RootWindow(disp, screen);
    else {
	win = XCreateSimpleWindow(disp, RootWindow(disp, screen), 0, 0,
				  200, 200, 1, 
				  WhitePixel( disp, screen ),
				  BlackPixel( disp, screen ));
	XStoreName(disp, win, argv[0]);
	XMapWindow(disp, win);
    }

    signal( SIGALRM, DoOneTurn );
    DoOneTurn();
    
    while(1){
    }
}

void DoOneTurn()
{
    DrawMessage( sys_errlist[ random()%NUMBERofERRORS ] );
    alarm( waitsec );
}

DrawMessage( message )
char *message;
{
    int text_width, text_height;
    int start;
    int i, j;
    int tmpi;
    Window tmpw;
    XCharStruct size_info;

    XGetGeometry( disp, win, &tmpw, &tmpi, &tmpi,
		 &win_width, &win_height, &tmpi, &tmpi );
    
    XClearWindow( disp, win );
    XSetForeground( disp, gc, WhitePixel( disp, screen ));
    XTextExtents( font_info, message, strlen( message ), &tmpi,
		 &tmpi, &tmpi, &size_info );

    text_width =  size_info.width;
    text_height = - size_info.ascent + size_info.descent;
    
    start = (win_width - text_width)/2;
    
    for(i=0; i<strlen(message); i++){
	XDrawString( disp, win,
		    gc, start, (win_height - text_height)/2,
		    message+i, 1 );
	start += XTextWidth( font_info, message+i, 1 );
	XFlush( disp );
	usleep(delay * 1000);
    }
}

error_exit(){
exit(1);}
